
subroutine CalcEnergy(Pos, rfix, itype, sigma2ls, epsls, Lx, Ly, Lz, rc, PEnergy, Dim, Ntypes, NAtom)
    implicit none
    integer, intent(in) :: Dim, NAtom, Ntypes
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos, rfix
    integer, intent(in), dimension(0:NAtom-1) :: itype
    real(8), intent(in), dimension(0:Ntypes-1,0:Ntypes-1) :: sigma2ls, epsls
    real(8), intent(in) :: Lx, Ly, Lz, rc
    real(8), intent(out) :: PEnergy

    integer :: i, j
    
    REAL(8), dimension(Dim) :: rijvec, POSI
    REAL(8), dimension(0:Ntypes-1,0:Ntypes-1) :: PEcutls
    REAL(8) :: rij_sq, rc_sq, sigmasq, eps !rij, 
    real(8), parameter :: k = 1000.0

    !Set value to zero
    PEnergy = 0.0
    !Square of cut-off distance:
    rc_sq = rc * rc
    !Building matrix of cut off PE values
    do i = 0, Ntypes - 1
        do j = 0, Ntypes - 1
            sigmasq = sigma2ls(i,j) !sigma^2 value for pair ij
            eps = epsls(i,j) !eps value for pair ij
			!Finding cutoff PE for pair ij:
            PEcutls(i,j) = 4.0*eps*((sigmasq/rc_sq)**6 - (sigmasq/rc_sq)**3) 
        enddo
    enddo 
    
    !Loop over particle pairs, adding to PEnergy
    do i = 0, NAtom - 1
        POSI = POS(i,:)
        do j = i + 1, NAtom - 1
            !Set sigma/eps values for selected particle pair i and j
            sigmasq = sigma2ls(itype(i),itype(j))
            eps = epsls(itype(i),itype(j))
            !LJ potential:
            !first find (r_ij)^2
            rijvec = POS(j,:) - POSI
            !PBC - nearest image convention:
            rijvec(1) = rijvec(1) - Lx*dnint(rijvec(1)/Lx)
            rijvec(2) = rijvec(2) - Ly*dnint(rijvec(2)/Ly)
            rijvec(3) = rijvec(3) - Lz*dnint(rijvec(3)/Lz)
		
            rij_sq = SUM(rijvec*rijvec)
            !Check if squared distance is less than squared cut-off radius:
            if (rij_sq > rc_sq) then
                cycle
            endif
		
            !finding potential energy, summing to value found thus far
            PEnergy = 4.0*eps*((sigmasq/rij_sq)**6 - (sigmasq/rij_sq)**3) - PEcutls(itype(i),itype(j)) + PEnergy     
        enddo

        !If ith particle is anchored to wall, we have to include this contribution to forces/energy:
        if (itype(i) == 0) then
            rijvec = POSI - rfix(i,:) !Find displacement for equilib. position
            !PBC - nearest image convention:
            rijvec(1) = rijvec(1) - Lx*dnint(rijvec(1)/Lx)
            rijvec(2) = rijvec(2) - Ly*dnint(rijvec(2)/Ly)
            rijvec(3) = rijvec(3) - Lz*dnint(rijvec(3)/Lz)
			!Find potential energy due to harmonic potential
            PEnergy = k*0.5*SUM(rijvec*rijvec) + PEnergy
        endif
    enddo    
end subroutine


subroutine CalcEnergyForces(Pos, rfix, itype, sigma2ls, epsls, Lx, Ly, Lz, rc, PEnergy, Forces, Dim, Ntypes, NAtom)
    implicit none
    integer, intent(in) :: Dim, NAtom, Ntypes
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos, rfix
    integer, intent(in), dimension(0:NAtom-1) :: itype
    real(8), intent(in), dimension(0:Ntypes-1,0:Ntypes-1) :: sigma2ls, epsls
    real(8), intent(in) :: Lx, Ly, Lz, rc
    real(8), intent(out) :: PEnergy
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Forces

    INTEGER :: i, j
    
    REAL(8), dimension(Dim) :: rijvec, POSI, Fij
    REAL(8), DIMENSION(0:Ntypes-1,0:Ntypes-1) :: PEcutls
    REAL(8) :: rij_sq, rc_sq, sigmasq, eps 
!f2py intent(in, out) :: Forces
    real(8), parameter :: k = 1000.0

    !Set value to zero
    PEnergy = 0.0
    Forces = 0.0
    !Square of cut-off distance:
    rc_sq = rc * rc
    !Building matrix of cut off PE values
    do i = 0, Ntypes - 1
        do j = 0, Ntypes - 1
            sigmasq = sigma2ls(i,j) !sigma^2 value for pair ij
            eps = epsls(i,j) !eps value for pair ij
			!Finding PE cutoff value for pair ij
            PEcutls(i,j) = 4.0*eps*((sigmasq/rc_sq)**6 - (sigmasq/rc_sq)**3) 
        enddo
    enddo 

    
    !Loop over particle pairs, adding to PEnergy, Forces array
    do i = 0, NAtom - 1
        POSI = POS(i,:)
        do j = i + 1, NAtom - 1
            !Set sigma/eps values for selected particle pair i/j
            sigmasq = sigma2ls(itype(i),itype(j))
            eps = epsls(itype(i),itype(j))
            !LJ potential:
            !first find (r_ij)^2
            rijvec = POS(j,:) - POSI
            !PBC - nearest image convention:
            rijvec(1) = rijvec(1) - Lx*dnint(rijvec(1)/Lx)
            rijvec(2) = rijvec(2) - Ly*dnint(rijvec(2)/Ly)
            rijvec(3) = rijvec(3) - Lz*dnint(rijvec(3)/Lz)
		
            rij_sq = SUM(rijvec*rijvec)
            !Check if squared distance is less than squared cut-off radius:
            if (rij_sq > rc_sq) then
                cycle
            endif
		
            !finding potential energy, summing to value found thus far
            PEnergy = 4.0*eps*((sigmasq/rij_sq)**(6) - (sigmasq/rij_sq)**(3)) - PEcutls(itype(i),itype(j)) + PEnergy 
            !finding forces
            Fij = -24.0*eps*(1.0/rij_sq)*(2.0*(sigmasq/rij_sq)**6 - (sigmasq/rij_sq)**3) * rijvec

            Forces(i,:) = Forces(i,:) + Fij
            Forces(j,:) = Forces(j,:) - Fij !By Newton's 3rd Law	        
        enddo

        !If ith particle is anchored to wall, we have to include this contribution to forces/energy:
        if (itype(i) == 0) then
            rijvec = POSI - rfix(i,:) !Find displacement for equilib. position
            !PBC - nearest image convention:
            rijvec(1) = rijvec(1) - Lx*dnint(rijvec(1)/Lx)
            rijvec(2) = rijvec(2) - Ly*dnint(rijvec(2)/Ly)
            rijvec(3) = rijvec(3) - Lz*dnint(rijvec(3)/Lz)

			!Find potential energy due to harmonic potential
            PEnergy = k*0.5*SUM(rijvec*rijvec) + PEnergy
			!Find forces on ith particle
            Fij = -k*rijvec
            Forces(i,:) = Forces(i,:) + Fij
        endif   
    enddo
end subroutine


subroutine VVIntegrate(Pos, Vel, Accel, dx, rfix, itype, sigma2ls, epsls, Lx, Ly, Lz, rc, dt, KEnergy, PEnergy, Dim, Ntypes, NAtom)
    implicit none
    integer, intent(in) :: Dim, Ntypes, NAtom
    real(8), intent(in) :: Lx, Ly, Lz, rc, dt
    real(8), intent(in), dimension(0:NAtom-1,0:Dim-1) :: rfix
    integer, intent(in), dimension(0:NAtom-1) :: itype
    real(8), intent(in), dimension(0:Ntypes-1,0:Ntypes-1) :: sigma2ls, epsls
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Pos, Vel, Accel, dx
!f2py intent(in,out) :: Pos, Vel, Accel, dx

    real(8), intent(out) :: KEnergy, PEnergy

    real(8), dimension(0:NAtom-1, 0:Dim-1) :: dxiter

    external :: CalcEnergyForces

    !Calculate next position
    dxiter = dt * Vel + 0.5 * dt*dt * Accel  
    Pos = Pos + dxiter
    !Periodic Boundary Conditions:
    Pos(:,0) = Pos(:,0) - Lx*dnint(Pos(:,0)/Lx - 0.5)
    Pos(:,1) = Pos(:,1) - Ly*dnint(Pos(:,1)/Ly - 0.5)
    Pos(:,2) = Pos(:,2) - Lz*dnint(Pos(:,2)/Lz - 0.5)

    !Add displacement vector to array (for later mean square displacement calc.)
    dx = dx + dxiter    

    !Propagate velocity by 1/2 step
    Vel = Vel + 0.5 * dt * Accel
    !Find forces at new position for updated acceleration
    call CalcEnergyForces(Pos, rfix, itype, sigma2ls, epsls, Lx, Ly, Lz, rc, PEnergy, Accel, Dim, Ntypes, NAtom)

    !Complete velocity using forces at new position
    Vel = Vel + 0.5 * dt * Accel
    KEnergy = 0.5 * sum(Vel*Vel)
end subroutine